/*
 * pc300util.c	Cyclades-PC300(tm) debug utility.
 *
 * Authors:	Daniela Squassoni <daniela@cyclades.com>
 * 		Alex Ura <alex@cyclades.com>
 * 		Henrique Gobbi <henrique@cyclades.com>
 *
 * Copyright:	(c) 1999-2001 Cyclades Corp.
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License
 *      as published by the Free Software Foundation; either version
 *      2 of the License, or (at your option) any later version.
 *
 * $Log: pc300.c,v $
 * Revision 0.3 2001/03/05 Daniela 
 * Implemented pc300util_end and pc300_config routines.
 * Implemented -H option, to get information about the PC300 hardware. 
 * 
 * Revision 0.2 2001/02/07 Daniela 
 * Printf's revision. 
 *
 * Revision 0.1 2000/12/22 Daniela, Alex, Henrique 
 * Initial version.
 *
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <linux/netdevice.h>
#include <net/if_arp.h>
#include <signal.h>
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netdb.h>

#include <linux/if_packet.h>
#include <linux/if_ether.h>

#include <time.h>

#include "pc300util.h"

/************************/
/*  PC300util routines  */
/************************/
 
void out_line(int lines){
	int i;
	
	for (i = 0;i < lines; i++) {
		fprintf(stdout, "\r\n");
	}
}

void out_on_off(ucchar flag){
	if(flag){
		fprintf(stdout, "ON ");
	} else {
		fprintf(stdout, "OFF");
	}
}  

int pc300_trace(void)
{
	unsigned char buffer[1500];
	int size;
	int j;
	struct sockaddr_ll saddr;
	int asize;
	struct ifreq ifr;

	if ((s = socket(AF_PACKET, SOCK_RAW, htons(ETH_P_CUST))) == -1) {
		perror("pc300trace: socket");
		return 1;
	}
	
	/* ioctl to read the interface number */
	memset (&ifr, 0, sizeof(struct ifreq));
	(void)strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));
	if (ioctl(s, SIOCGIFINDEX, (char *)&ifr) != 0) {
		perror("pc300trace: ioctl 1");
	}
	
	/* Bind to the interface name */
	memset (&saddr, 0, sizeof(struct sockaddr_ll));
	saddr.sll_family = AF_PACKET;
	saddr.sll_protocol = htons(ETH_P_CUST);
	saddr.sll_ifindex = ifr.ifr_ifindex;
	if (bind(s, (struct sockaddr *)&saddr, sizeof(saddr))) {
		perror("pc300trace: bind");
	}
	
	/* ioctl to activate the trace */
	trace_on = 1;
	memset (&ifr, 0, sizeof(struct ifreq));
	(void)strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));
	ifr.ifr_data = &trace_on;
	if (ioctl(s, SIOCSPC300TRACE, (char *)&ifr) != 0) {
		perror("pc300trace: ioctl 2");
	}

	fprintf(stdout, "\r\nPC300: %s trace on", device);
	fprintf(stdout, "\r\nPress <Ctrl c> to stop\r\n");
	
	for (;;) {
		asize = sizeof(saddr);
	
		if ((size = recvfrom(s, buffer, sizeof(buffer), 0, 
				(struct sockaddr *)&saddr, &asize)) == -1) {
			perror("pc300trace: recv");
			return 1;
		}
		j = sizeof(saddr);
		if (getsockname (s, (struct sockaddr *)&saddr, &j) < 0) {	
			perror("pc300trace: getsockname");
			return 1;
		}
		fprintf(stdout, "\r\n%c", buffer[0]);
		fprintf(stdout, "%c", buffer[1]);
		fprintf(stdout, "%c", buffer[2]);
		hex_dump(&buffer[3], (size - 3));

		putc('\n', stdout);
		fflush(stdout);
	}
}

void hex_dump(unsigned char *data, int length)
{
	int  i, j, length2;
	unsigned char c;
	char *data2;

	length2 = length;
	data2   = data;
	
	for (i = 0; length > 0; i += 16) {
		for (j = 0; j < 16; j++) {
			c = *data2++;
			length2--;
			
			if (length2 >= 0)
				fprintf(stdout, "%2.2X ", c);
			else
				fprintf(stdout, "   ");
		}

		fprintf(stdout, " | ");
				
		for (j = 0; j < 16 && length > 0; j++) {
 			c = *data++;
 			length--;
 			
			if (c >= 0x20 && c < 0x7F)
				putc(c, stdout);
			else
				putc('.', stdout);
		}
		
		putc('\n', stdout);
	}
}

void pc300_config(void)
{
	out_line(1);
	fprintf(stdout, "--------------- PC300 - CONFIGURATION --------------");
	out_line(2);
	fflush(NULL);
	strcat(pc300cfg_string, device);
	if (options_mask & DO_FILE) {
		strcat(pc300cfg_string, " |tee > ");
	        strcat(pc300cfg_string, pc300_file);
	}
	system(pc300cfg_string);
	out_line(1);
	fflush(NULL);
}

void pc300_stats(int sk)
{
	pc300stats_t pc300_stats;
	struct ifreq ifr; 
	
	memset (&ifr, 0, sizeof(struct ifreq));
	(void)strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));
	ifr.ifr_data = (char *)&pc300_stats;

	if (ioctl(sk, SIOCGPC300UTILSTATS, &ifr) < 0) {
		printf("PC300 stats ioctl err\n");
		pc300util_end (-1);
	}

	/* printing data from pc300_stats struct */
	/* /usr/src/linux/drivers/net/pc300.h    */
	out_line(1);
	fprintf(stdout, "---------------- PC300 - STATISTICS ----------------");
	out_line(2);

	fprintf(stdout, "device: %s ", device);
	out_line(2);
	fprintf(stdout, "hw type:");
	switch(pc300_stats.hw_type){
		case PC300_RSV:
			fprintf(stdout, "RSV");
			break;
		case PC300_X21:
			fprintf(stdout, "X21");
			break;
		case PC300_TE:
			fprintf(stdout, "TE");
			break;
	}

	fprintf(stdout, "  line on:%lu",pc300_stats.line_on);
	fprintf(stdout, "  line off:%lu",pc300_stats.line_off);
	fprintf(stdout, "  multicast:%lu", pc300_stats.gen_stats.multicast);
	fprintf(stdout, "  collisions:%lu", pc300_stats.gen_stats.collisions);

	out_line(2);

	fprintf(stdout, "rx : pack:%lu",pc300_stats.gen_stats.rx_packets);
	fprintf(stdout, "  bytes:%lu",pc300_stats.gen_stats.rx_bytes);
	fprintf(stdout, "  errors:%lu",pc300_stats.gen_stats.rx_errors);
	fprintf(stdout, "  drops:%lu",pc300_stats.gen_stats.rx_dropped);
	fprintf(stdout,"  compressed:%lu",pc300_stats.gen_stats.rx_compressed);

	out_line(1);

	fprintf(stdout, "     errors : ");
	fprintf(stdout, "length:%lu", pc300_stats.gen_stats.rx_length_errors);
	fprintf(stdout, "  overrun:%lu", pc300_stats.gen_stats.rx_over_errors);
	fprintf(stdout, "  crc:%lu", pc300_stats.gen_stats.rx_crc_errors);
	fprintf(stdout, "  frame:%lu", pc300_stats.gen_stats.rx_frame_errors);
	fprintf(stdout, "  fifo:%lu", pc300_stats.gen_stats.rx_fifo_errors);
	fprintf(stdout, "  missed:%lu", pc300_stats.gen_stats.rx_missed_errors);
	
	out_line(2);

	fprintf(stdout, "tx : pack:%lu",pc300_stats.gen_stats.tx_packets);
	fprintf(stdout, "  bytes:%lu",pc300_stats.gen_stats.tx_bytes);
	fprintf(stdout, "  errors:%lu",pc300_stats.gen_stats.tx_errors);
	fprintf(stdout, "  drops:%lu",pc300_stats.gen_stats.tx_dropped);
	fprintf(stdout,"  compressed:%lu",pc300_stats.gen_stats.tx_compressed);
	
	out_line(1);

	fprintf(stdout, "     errors : ");
	fprintf(stdout, "abort:%lu", pc300_stats.gen_stats.tx_aborted_errors);
	fprintf(stdout,"  carrier:%lu",pc300_stats.gen_stats.tx_carrier_errors);
	fprintf(stdout,"  heartbeat:%lu",pc300_stats.gen_stats.tx_fifo_errors);
	fprintf(stdout,"  window:%lu", pc300_stats.gen_stats.tx_window_errors);

	if(pc300_stats.hw_type == PC300_TE){
		out_line(2);
		fprintf(stdout, "TE statistic");
	  
		out_line(1);
		fprintf(stdout, "pden :%10u", pc300_stats.te_stats.pden);
		fprintf(stdout, "      los  :%10u", pc300_stats.te_stats.los);
		fprintf(stdout, "      losr :%10u", pc300_stats.te_stats.losr);
	  
		out_line(1);
		fprintf(stdout, "lfa  :%10u", pc300_stats.te_stats.lfa);
		fprintf(stdout, "      farec:%10u", pc300_stats.te_stats.farec);
		fprintf(stdout, "      lmfa :%10u", pc300_stats.te_stats.lmfa);
	  
		out_line(1);
		fprintf(stdout, "ais  :%10u", pc300_stats.te_stats.ais);
		fprintf(stdout, "      sec  :%10u", pc300_stats.te_stats.sec);
		fprintf(stdout, "      es   :%10u", pc300_stats.te_stats.es);
	  
		out_line(1);
		fprintf(stdout, "rai  :%10u", pc300_stats.te_stats.rai);
		fprintf(stdout, "      bec  :%10u", pc300_stats.te_stats.bec);
		fprintf(stdout, "      fec  :%10u", pc300_stats.te_stats.fec);
	  
		out_line(1);
		fprintf(stdout, "cvc  :%10u", pc300_stats.te_stats.cvc);
		fprintf(stdout, "      cec  :%10u", pc300_stats.te_stats.cec);
		fprintf(stdout, "      ebc  :%10u", pc300_stats.te_stats.ebc);

		out_line(2);
		fprintf(stdout, "Alarms : ");
		fprintf(stdout, "Red    : ");
		out_on_off(pc300_stats.te_stats.red_alarm);
		fprintf(stdout, "     Blue  : ");
		out_on_off(pc300_stats.te_stats.blue_alarm);

		out_line(1);
		fprintf(stdout, "         Yellow : ");
		out_on_off(pc300_stats.te_stats.yellow_alarm);
		fprintf(stdout, "     LFA   : ");
		out_on_off(pc300_stats.te_stats.loss_fa);
	  
		out_line(1);
		fprintf(stdout, "         LMFA   : ");
		out_on_off(pc300_stats.te_stats.loss_mfa);
		fprintf(stdout, "     prbs  : ");
		out_on_off(pc300_stats.te_stats.prbs);
	}
	out_line(1);
}

void pc300_status(int sk)
{
	pc300status_t pc300_status;
	struct ifreq ifr; 
	
	memset (&pc300_status, 0, sizeof(pc300status_t));
	memset (&ifr, 0, sizeof(struct ifreq));
	(void)strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));
	ifr.ifr_data = (char *)&pc300_status;

	if (ioctl(sk, SIOCGPC300UTILSTATUS, &ifr) < 0) {
		printf("\r\nPC300 status ioctl error\n");
		pc300util_end (-1);
	}

	out_line(1);
	fprintf(stdout, "------------------ PC300 - STATUS ------------------");
	out_line(2);

	fprintf(stdout, "device: %s ", device);
	out_line(2);
	fprintf(stdout, "Hardware Type: ");
	switch(pc300_status.hw_type){
		case PC300_RSV:
			fprintf(stdout, "RSV");
			break;
		case PC300_X21:
			fprintf(stdout, "X21");
			break;
		case PC300_TE:
			fprintf(stdout, "TE");
			break;
	}

	out_line(2);
	if (pc300_status.hw_type == PC300_TE){
		fprintf(stdout, "TE Status: ");
		fprintf(stdout, "  sync:                          ");
		out_on_off(pc300_status.te_status.sync);
		fprintf(stdout, "\r\n             Red Alarm:                     ");
		out_on_off(pc300_status.te_status.red_alarm);
		fprintf(stdout, "\r\n             Blue Alarm:                    ");
		out_on_off(pc300_status.te_status.blue_alarm);
		fprintf(stdout, "\r\n             Loss of Frame alignment:       ");
		out_on_off(pc300_status.te_status.loss_fa);
		fprintf(stdout, "\r\n             Yellow Alarm:                  ");
		out_on_off(pc300_status.te_status.yellow_alarm);
		fprintf(stdout, "\r\n             Loss of Multi Frame alignment: ");
		out_on_off(pc300_status.te_status.loss_mfa);
		fprintf(stdout, "\r\n             Prbs:                          ");
		out_on_off(pc300_status.te_status.prbs);
	} else {
		fprintf(stdout, "General: ");
		fprintf(stdout, "    DCD : ");
		out_on_off(pc300_status.gen_status.dcd);
		/* DSR is not available in PC300 */
		fprintf(stdout, "\r\n             DSR : N/A"); 
		fprintf(stdout, "\r\n             CTS : ");
		out_on_off(pc300_status.gen_status.cts);
		fprintf(stdout, "\r\n             RTS : ");
		out_on_off(pc300_status.gen_status.rts);
		fprintf(stdout, "\r\n             DTR : ");
		out_on_off(pc300_status.gen_status.dtr);
	}
	out_line(1);
}

int pc300_check_t1e1(int sk)
{
	pc300status_t pc300_status;
	struct ifreq ifr; 
	
	memset (&pc300_status, 0, sizeof(pc300status_t));
	memset (&ifr, 0, sizeof(struct ifreq));
	(void)strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));
	ifr.ifr_data = (char *)&pc300_status;

	if (ioctl(sk, SIOCGPC300UTILSTATUS, &ifr) < 0) {
		printf("\r\nPC300 check status ioctl error\n");
		pc300util_end (-1);
	}

	if (pc300_status.hw_type != PC300_TE){
		fprintf(stdout, "\r\nInvalid hardware type. 
				 \r\nThis test is only for PC300 TE boards.\n");
		pc300util_end (-1);
	}
	if (pc300_status.te_status.red_alarm) {
		return 2;
	}
	if (pc300_status.te_status.prbs == 2) {
		return 1;
	}
	return 0;
}

void pc300_loopback (int sk, char loop_type)
{
	pc300loopback_t pc300_loopback;
	struct ifreq ifr; 
	
	memset (&pc300_loopback, 0, sizeof(pc300loopback_t));
	pc300_loopback.loop_type = loop_type;
	pc300_loopback.loop_on = 1;

	memset (&ifr, 0, sizeof(struct ifreq));
	(void)strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));
	ifr.ifr_data = (char *)&pc300_loopback;

	if (ioctl(sk, SIOCSPC300LOOPBACK, &ifr) < 0) {
		printf("PC300 loopback ioctl err\n");
		pc300util_end (-1);
	}
	gl_loop_type = loop_type;
	fprintf(stdout, "\r\nPC300: %s loop on", device);
	fprintf(stdout, "\r\nPress <Ctrl c> to stop\r\n");
	fflush(NULL);
	for (;;) {
	}
}

void pc300_pattern (int sk, char patrntst_on)
{
	pc300patterntst_t pc300_pattern;
	struct ifreq ifr; 
	int t1e1_status;
	
	t1e1_status = pc300_check_t1e1(sk);
	if (t1e1_status == 2) {
		fprintf(stdout, "\r\nLoss of signal: pattern test aborted.\n");
		pc300util_end (-1);
	}
	
	memset (&pc300_pattern, 0, sizeof(pc300patterntst_t));
	pc300_pattern.patrntst_on = patrntst_on;

	memset (&ifr, 0, sizeof(struct ifreq));
	(void)strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));
	ifr.ifr_data = (char *)&pc300_pattern;

	if (ioctl(sk, SIOCSPC300PATTERNTEST, &ifr) < 0) {
		printf("PC300 pattern test ioctl err\n");
		pc300util_end (-1);
	}
	if (!patrntst_on) {
		gl_patterntst_on = 0;
	} else {
		gl_patterntst_on = 1;
		fprintf(stdout, "\r\nPC300: %s Starting pattern test", device);
		fprintf(stdout, "\r\nPress <Ctrl c> to stop\r\n");
	
		fflush(NULL);
		for (;;) {
			sleep (1);
			t1e1_status = pc300_check_t1e1(sk);
			pc300_pattern.patrntst_on = 2;
			if (t1e1_status == 2) {
				/* Inactivates pattern test */
				pc300_pattern.patrntst_on = 0;
			}
			
			/* Starting pattern test monitor */
			if (ioctl(sk, SIOCSPC300PATTERNTEST, &ifr) < 0) {
				printf("PC300 pattern test ioctl err\n");
				pc300util_end (-1);
			}
			if (pc300_pattern.patrntst_on == 0) {
				fprintf(stdout, "\r\nLoss of signal: pattern test aborted.\n");
				pc300util_end (-1);
			}
		
			fprintf(stdout, "\r\n Status:  ");
			if (t1e1_status == 1) {
				fprintf(stdout, "Synchronized      ");
			} else {
				fprintf(stdout, "Not synchronized  ");
			}
			
			fprintf(stdout, " Pattern test errors: %-10u ", 
					pc300_pattern.num_errors);
			fflush(NULL);
		}
	}
}

void pc300_genloop (int sk, char loop_type)
{
	pc300loopback_t pc300_loopback;
	struct ifreq ifr; 
	int i = 15;
	
	memset (&pc300_loopback, 0, sizeof(pc300loopback_t));
	pc300_loopback.loop_type = loop_type;
	pc300_loopback.loop_on = 1;

	memset (&ifr, 0, sizeof(struct ifreq));
	(void)strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));
	ifr.ifr_data = (char *)&pc300_loopback;

	/* Starting the 15-second signal */
	if (ioctl(sk, SIOCSPC300LOOPBACK, &ifr) < 0) {
		printf("PC300 generate loop signal ioctl err\n");
		pc300util_end (-1);
	}
	gl_loop_type = loop_type;
	
	if (loop_type == PC300GENLOOPUP) {
		fprintf(stdout, "\r\nPC300: %s generating loop on signal", 
				device);
	} else {
		fprintf(stdout, "\r\nPC300: %s generating loop off signal", 
				device);
	}
	
	fprintf(stdout, "\r\nThis will take a few seconds...\n");
	fprintf(stdout, "\r\nTime remaining: %-2ds", i--);
	fflush(NULL);
	while (i >= 0) {
		sleep (1);
		fprintf(stdout, "\b\b\b%-2ds", i);
		fflush(NULL);
		i--;
	}

	/* Stopping the 15-second signal */
	pc300_loopback.loop_on = 0;
	if (ioctl(sk, SIOCSPC300LOOPBACK, &ifr) < 0) {
		printf("PC300 generate loop signal ioctl err\n");
		pc300util_end (-1);
	}
	fprintf(stdout, "\rDone.               \n");
	
	if (loop_type == PC300GENLOOPUP) {
		pc300_pattern (sk, 1);
	} else {
		gl_loop_type = 0;
	}
}

void pc300_hardware (int sk)
{
	pc300conf_t pc300_conf;
	pc300hw_t *pc300_hw;
	struct ifreq ifr; 
	
	memset (&pc300_conf, 0, sizeof(pc300conf_t));

	memset (&ifr, 0, sizeof(struct ifreq));
	(void)strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));
	ifr.ifr_data = (char *)&pc300_conf;

	/* Starting the 15-second signal */
	if (ioctl(sk, SIOCGPC300CONF, &ifr) < 0) {
		printf("PC300 hardware ioctl err\n");
		pc300util_end (-1);
	}
	pc300_hw = &pc300_conf.hw;

	out_line(1);
	fprintf(stdout, "----------------- PC300 - HARDWARE -----------------");
	out_line(2);
	fprintf (stdout, "- %s: PC300/", device);
	switch (pc300_hw->type) {
		case PC300_TE:
			fprintf (stdout, "TE ");
			break;

		case PC300_X21:
			fprintf (stdout, "X21 ");
			break;

		case PC300_RSV:
		default:
			fprintf (stdout, "RSV ");
			break;
	}
	fprintf (stdout, "\r\n  - %ldkB of RAM at 0x%08lx ", 
		pc300_hw->ramsize/1024, pc300_hw->ramphys);
	fprintf (stdout, "\r\n  - IRQ: %d", pc300_hw->irq); 
	fprintf (stdout, "\r\n  - Number of channels: %d", pc300_hw->nchan);
	fprintf (stdout, "\r\n  - CPLD version: %d", pc300_hw->cpld_id);
	out_line(2);
	fflush(NULL);
}

int
main(int argc, char **argv)
{
	register int cnt, op;
	register char *cp, *infile, *RFileName, *WFileName;
	RETSIGTYPE (*oldhandler)(int);

	cnt = -1;
	infile = NULL;
	RFileName = NULL;
	WFileName = NULL;
	if ((cp = strrchr(argv[0], '/')) != NULL) {
		program_name = cp + 1;
	} else {
		program_name = argv[0];
	}

	opterr = 0;
	
	(void)signal(SIGTERM, cleanup);
	(void)signal(SIGINT, cleanup);
	/* nohup (1) */
	if ((oldhandler = signal(SIGHUP, cleanup)) != SIG_DFL) {
		(void)signal(SIGHUP, oldhandler);
	}
	
	while ((op = getopt(argc, argv, "csSlrpUDPtHd:f:h")) != EOF) {
		switch (op) {

		case 'c':
			options_mask = options_mask | DO_CONFIG;
			break;

		case 'd':
			(void)strncpy(device, optarg, sizeof(device));
			break;

		case 's':
			options_mask = options_mask | DO_STATISTICS;
			break;

		case 'S':
			options_mask = options_mask | DO_STATUS;
			break;

		case 'l':
			options_mask = options_mask | DO_LOCLOOP;
			break;

		case 'r':
			options_mask = options_mask | DO_REMLOOP;
			break;

		case 'p':
			options_mask = options_mask | DO_PAYLOOP;
			break;

		case 'U':
			options_mask = options_mask | DO_LOOPUP;
			break;

		case 'D':
			options_mask = options_mask | DO_LOOPDOWN;
			break;

		case 'P':
			options_mask = options_mask | DO_PATTERN;
			break;

		case 't':
			options_mask = options_mask | DO_TRACE;
			break;

		case 'H':
			options_mask = options_mask | DO_HARDWARE;
			break;

		case 'f':
			options_mask = options_mask | DO_FILE;
		        pc300_file = optarg;	
			break;

		case 'h':
			options_mask = options_mask | DO_HELP;
			break;

		default:
			puts(helptext);
			pc300util_end (0);
			/* NOTREACHED */
		}
	}
	exec_action(options_mask);	
	
	if ((options_mask & DO_FILE) && stdout) {
		fclose(stdout);
	}
	if (skfd >=0) {
		(void) close (skfd);
	}
	if (s >=0) {
		(void) close (s);
	}
	return (0);
}

void exec_action(int action)
{
	char loop_type = 0;
	
	if (action & DO_HELP){
		puts(helptext_detailed);
		return;
	}

	if (action & MASK_OPENSOCK) {
		/* Create a channel to the NET kernel. */
		if ((skfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
			printf("\r\nsocket err");
			exit(1);
   		}
	}

	if (action & DO_FILE){
		if ((stdout = fopen(pc300_file, "w+")) == NULL) {
			printf("\r\nfile open err");
			pc300util_end (-1);
		}
	}

	if (action & DO_CONFIG) {
		pc300_config();
	}

	if (action & DO_STATISTICS){
		pc300_stats(skfd);
	}

	if (action & DO_STATUS){
		pc300_status(skfd);
	}
			
	if (action & DO_LOCLOOP){ 
		loop_type = PC300LOCLOOP;
		pc300_loopback (skfd, loop_type);
	}

	if (action & DO_REMLOOP){
		loop_type = PC300REMLOOP;
		pc300_loopback (skfd, loop_type);
	}

	if (action & DO_PAYLOOP){ 
		loop_type = PC300PAYLOADLOOP;
		pc300_loopback (skfd, loop_type);
	}

	if (action & DO_LOOPUP){
		loop_type = PC300GENLOOPUP;
		pc300_genloop (skfd, loop_type);
	}

	if (action & DO_LOOPDOWN){
		loop_type = PC300GENLOOPDOWN;
		pc300_genloop (skfd, loop_type);
	}

	if (action & DO_PATTERN){
		pc300_pattern (skfd, 1);
	}
	
	if (action & DO_HARDWARE){
		pc300_hardware (skfd);
	}
			
	if (action & DO_TRACE){
		pc300_trace();
	}
}

void pc300util_end (int err)
{
	if ((options_mask & DO_FILE) && stdout) {
		fclose(stdout);
	}
	if (skfd >=0) {
		(void) close (skfd);
	}
	if (s >=0) {
		(void) close (s);
	}
	exit (err);
}

RETSIGTYPE
cleanup (int signo)
{
	struct ifreq ifr;
	pc300loopback_t pc300_loopback;

	/* Dumb access to signo, just to please GCC ... */
	signo = signo;
	
	fflush(NULL);
	if (trace_on) {
		fprintf(stdout, "\r\nDeactivating trace...\r\n");
		/* ioctl para ativar o trace */
		trace_on = 0;
		memset (&ifr, 0, sizeof(struct ifreq));
		(void)strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));
		ifr.ifr_data = &trace_on;
		if (ioctl(s, SIOCSPC300TRACE, (char *)&ifr) != 0) {
			perror("pc300trace: ioctl");
		}
	}
	if (gl_patterntst_on) {
		fprintf(stdout, "\r\nDeactivating pattern test...\r\n");
		pc300_pattern (skfd, 0);
	}
	if (gl_loop_type) {
		fprintf(stdout, "\r\nDeactivating loopback...\r\n");
	
		pc300_loopback.loop_type = gl_loop_type;
		pc300_loopback.loop_on = 0;

		memset (&ifr, 0, sizeof(struct ifreq));
		(void)strncpy(ifr.ifr_name, device, sizeof(ifr.ifr_name));
		ifr.ifr_data = (char *)&pc300_loopback;

		if (ioctl(skfd, SIOCSPC300LOOPBACK, (char *)&ifr) < 0) {
			printf("PC300 loopback ioctl err\n");
			pc300util_end (-1);
		}
		if ((gl_loop_type == PC300GENLOOPUP) || 
			(gl_loop_type == PC300GENLOOPDOWN)) {
			pc300_genloop (skfd, PC300GENLOOPDOWN);
		}
	}
	pc300util_end (0);
}

